unit U1PlugTest;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ComCtrls, StdCtrls, ShellAPI;

type

  //: Tipo para ejecutar la rutina principal de los plug-ins.
  TExecuteRoutine = procedure (Sender:TObject) of object;

  // Clase para el form.
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    I1: TMenuItem;
    File1: TMenuItem;
    Ediin1: TMenuItem;
    PLugIns1: TMenuItem;
    Memo1: TMemo;
    StatusBar1: TStatusBar;
    N1: TMenuItem;
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure I1Click(Sender: TObject);
    procedure StatusBar1Click(Sender: TObject);
  private
    { Private declarations }
    _pathP:String;
    _FilesList:TStringList;

    // Click en un elemento del menu de plugins.
    procedure _ClickElement (Sender: TObject);
    // mostrar mensaje
    procedure _mens(Str:String);
  public
    { Public declarations }
  end;

// constantes varias...
const
  PLUG_DIRNAME = 'plug-ins';
  PLUG_MASK = '*.bpl';
  CHAR_BACKSLASH = '\';
  MASK_ALL_FILES = '*.*';
  CHAR_POINT = '.';
  PLUG_MAIN_FORM = 'TFormMainPlug';

var
  Form1: TForm1;


implementation

{$R *.dfm}


//
procedure FindFiles(StartDir, FileMask: string; recursively: boolean; var FilesList: TStringList);
var
  SR: TSearchRec;
  DirList: TStringList;
  IsFound: Boolean;
  i: integer;
begin
  if (StartDir[length(StartDir)] <> CHAR_BACKSLASH) then begin
    StartDir := StartDir + CHAR_BACKSLASH;
  end;

  // Crear la lista de ficheos en el directorio StartDir (no directorios!)
  IsFound := FindFirst(StartDir + FileMask, faAnyFile - faDirectory, SR) = 0;
  // MIentras encuentre
  while IsFound do  begin
    FilesList.Add(StartDir + SR.Name);
    IsFound := FindNext(SR) = 0;
  end;

  FindClose(SR);

  // Recursivo?
  if (recursively) then begin
    // Build a list of subdirectories
    DirList := TStringList.Create;
    // proteccion
    try
      IsFound := FindFirst(StartDir + MASK_ALL_FILES, faAnyFile, SR) = 0;
      while IsFound do
      begin
        if ((SR.Attr and faDirectory) <> 0) and
          (SR.Name[1] <> CHAR_POINT) then
          DirList.Add(StartDir + SR.Name);
        IsFound := FindNext(SR) = 0;
      end;
      FindClose(SR);

      // Scan the list of subdirectories
      for i := 0 to DirList.Count - 1 do
        FindFiles(DirList[i], FileMask, recursively, FilesList);
    finally
      DirList.Free;
    end;
  end;
end;

//


procedure TForm1.FormShow(Sender: TObject);
var
  Str, cName:String;
  i:Integer;
begin

  // Ocultar el men
  PLugIns1.Visible := False;
  _FilesList := TStringList.Create();

  // Detectando plugins
  _mens('Directorio de plug-ins...');
  _pathP := IncludeTrailingBackSlash(ExtractFilePath(Application.ExeName));
  _pathP := IncludeTrailingBackSlash(_pathP + PLUG_DIRNAME);

  // Crearlo si no existe
  ForceDirectories(_pathP);
  _mens('Creando directorio si es necesario...');

  // Buscar plugIns
  _mens('Buscando plug-ins instalados...');  
  FindFiles(_pathP, PLUG_MASK, False, _FilesList);

  // No encontrados
  if (_FilesList.Count = 0) then begin
    _mens('No se han encontrado plug-ins.');
    Exit;
  end
  else begin
    PLugIns1.Visible := true;
    _mens(Format('Encontrados %d plug-ins.',[_FilesList.Count]));
  end;

  // Cargarlos...
  for i := 0 to (_FilesList.Count - 1) do begin
    // Aadir los nombres...
    Str := ExtractFileName(_FilesList.Strings[i]);
    Str := ChangeFileExt(Str, '');
    // menu
    PLugIns1.Add(NewItem(Str, 0, False, True, _ClickElement, 0, Str));
    _mens('Encontrado plug-in: ' + _FilesList.Strings[i]);
  end;

end;

procedure TForm1._mens(Str: String);
begin
  Memo1.Lines.Add(Str);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  // Asignado?
  if Assigned(_FilesList) then begin
    FreeAndNil(_FilesList);
  end;
end;

procedure TForm1._ClickElement(Sender: TObject);
var
  Str:String;
  hndl:HMODULE;
  AClass:TPersistentClass;
  AForm:TForm;
  Routine: TMethod;
begin

  // Cual es el elemento pulsado?
  Str := TMenuItem(Sender).Name;
  // Buscar el fichero
  Str := Str + '.bpl';
  Str := _pathP + Str;
  // Existe?
  if FileExists(Str) then begin
    // Cargar
    hndl := LoadPackage(Str);

    // Accder a la clase del menu
    AClass := GetClass(PLUG_MAIN_FORM);

    // Bloque de proteccion para la carga
    try
      // No ha podido acceder a ella
      if (AClass <> nil) then begin
        AForm := TComponentClass(AClass).Create(Application) as TForm;

        // proteccion para liberar
        try
          // Acceso a la rutina
          Routine.Data := Pointer(AForm);
          // Returns the address of a published method.
          Routine.Code := (AForm).MethodAddress('Execute');

          // Encontrada la rutina de acceso
          if (Routine.Code = nil) then begin
            MessageDlg('Error al cargar el plug-in', mtError, [mbOK], 0);MessageDlg('Error, no se ha encontrado el procedimiento (Execute) para el correcto funcionamieno del Plug-in.', mtError, [mbOK], 0);
            Exit;
          end;

          // ejecutar
          TExecuteRoutine(Routine)(Self);

        finally
          Aform.Free;
        end;

        // Liberar / Descargar
        UnloadPackage(hndl);
      end
      else begin
        MessageDlg('La clase para acceder al plug-in parece que no est correctamente registrada.', mtError, [mbOK], 0);
      end;
    except
      on E:Exception do begin
        MessageDlg('Error al cargar el plug-in.', mtError, [mbOK], 0);
      end;
    end;
  end;
end;

procedure TForm1.I1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.StatusBar1Click(Sender: TObject);
begin
    ShellExecute(Handle,
             'open',
             'http://neftali.clubdelphi.com',
             nil,
             nil,
             SW_SHOW);
end;

end.

